library design;

import 'package:design/color_extensions.dart';
import 'package:flutter/material.dart';

class AppPrimaryButton extends StatelessWidget {
  const AppPrimaryButton(
    this.title, {
    this.onPressed,
    this.onLongPress,
    this.onHover,
    this.onFocusChange,
    this.focusNode,
    this.autofocus = false,
    this.clipBehavior = Clip.none,
    this.statesController,
    this.backgroundColor,
    this.rippleColor,
    this.rippleColorLightenByPercent = 33,
    this.disabledBackgroundColor,
    this.foregroundColor,
    this.disabledForegroundColor,
    this.shadowColor,
    this.elevationDefault = 0,
    this.elevationPressed = 0,
    this.minimumSize = const Size(192, 48),
    this.fixedSize,
    this.maximumSize,
    this.splashFactory,
    this.side,
    this.shape,
    this.enabledMouseCursor,
    this.disabledMouseCursor,
    this.visualDensity,
    this.tapTargetSize,
    this.animationDuration,
    this.enableFeedback,
    this.alignment,
    this.padding = const EdgeInsets.symmetric(horizontal: 16, vertical: 8),
    this.textStyle = const TextStyle(
      fontSize: 16,
      color: Color(0xFFFFFFFF),
    ),
    Key? key,
  }) : super(key: key);

  final VoidCallback? onPressed;
  final VoidCallback? onLongPress;
  final ValueChanged<bool>? onHover;
  final ValueChanged<bool>? onFocusChange;
  final String title;
  final double elevationDefault;
  final double elevationPressed;
  final FocusNode? focusNode;
  final bool autofocus;
  final Clip clipBehavior;
  final MaterialStatesController? statesController;

  final Color? backgroundColor;
  final Color? rippleColor;
  final double rippleColorLightenByPercent;
  final Color? disabledBackgroundColor;
  final Color? foregroundColor;
  final Color? disabledForegroundColor;
  final Color? shadowColor;
  final TextStyle textStyle;
  final EdgeInsets padding;
  final Size minimumSize;
  final Size? fixedSize;
  final Size? maximumSize;
  final BorderSide? side;
  final OutlinedBorder? shape;
  final MouseCursor? enabledMouseCursor;
  final MouseCursor? disabledMouseCursor;
  final VisualDensity? visualDensity;
  final MaterialTapTargetSize? tapTargetSize;
  final Duration? animationDuration;
  final bool? enableFeedback;
  final AlignmentGeometry? alignment;
  final InteractiveInkFeatureFactory? splashFactory;

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      onHover: onHover,
      focusNode: focusNode,
      onFocusChange: onFocusChange,
      onLongPress: onLongPress,
      autofocus: autofocus,
      clipBehavior: clipBehavior,
      statesController: statesController,
      onPressed: onPressed,
      style: ElevatedButton.styleFrom(
        backgroundColor: backgroundColor,
        disabledBackgroundColor: disabledBackgroundColor,
        foregroundColor: foregroundColor,
        disabledForegroundColor: disabledForegroundColor,
        shadowColor: shadowColor,
        textStyle: textStyle,
        padding: padding,
        minimumSize: minimumSize,
        fixedSize: fixedSize,
        maximumSize: maximumSize,
        enabledMouseCursor: enabledMouseCursor,
        disabledMouseCursor: disabledMouseCursor,
        visualDensity: visualDensity,
        tapTargetSize: tapTargetSize,
        animationDuration: animationDuration,
        enableFeedback: enableFeedback,
        alignment: alignment,
        splashFactory: splashFactory,
        side: side ?? BorderSide.none,
        shape: shape ??
            const RoundedRectangleBorder(
              side: BorderSide.none,
              borderRadius: BorderRadius.all(Radius.circular(8)),
            ),
      ).copyWith(
        elevation: MaterialStateProperty.resolveWith((states) {
          if (states.contains(MaterialState.pressed)) {
            return elevationPressed;
          }
          return elevationDefault;
        }),
        overlayColor: MaterialStateProperty.resolveWith(
          (states) {
            if (states.contains(MaterialState.pressed)) {
              final primaryColor = Theme.of(context).colorScheme.primary;
              final overlay = rippleColor ?? backgroundColor ?? primaryColor;
              return overlay.lighten(percent: rippleColorLightenByPercent);
            }
            return null;
          },
        ),
      ),
      child: Text(
        title,
        style: textStyle,
      ),
    );
  }
}
